#ifdef GL_ES
precision lowp float;
#endif

varying vec2 v_texCoord;
uniform sampler2D u_texture;

vec3 RGBToHSL(vec3 color)
{
	vec3 hsl;
	
	float fmin = min(min(color.r, color.g), color.b);    //Min. value of RGB
	float fmax = max(max(color.r, color.g), color.b);    //Max. value of RGB
	float delta = fmax - fmin;             //Delta RGB value
    
	hsl.z = (fmax + fmin) * 0.5; // Luminance
    
	if (delta == 0.0)		//This is a gray, no chroma...
	{
		hsl.x = 0.0;	// Hue
		hsl.y = 0.0;	// Saturation
	}
	else //Chromatic data...
	{
		if (hsl.z < 0.5)
			hsl.y = delta / (fmax + fmin); // Saturation
		else
			hsl.y = delta / (2.0 - fmax - fmin); // Saturation
		
		float deltaR = (((fmax - color.r) * 0.1666) + (delta * 0.5)) / delta;
		float deltaG = (((fmax - color.g) * 0.1666) + (delta * 0.5)) / delta;
		float deltaB = (((fmax - color.b) * 0.1666) + (delta * 0.5)) / delta;
        
		if (color.r == fmax )
			hsl.x = deltaB - deltaG; // Hue
		else if (color.g == fmax)
			hsl.x = (0.3333) + deltaR - deltaB; // Hue
		else if (color.b == fmax)
			hsl.x = (0.6666) + deltaG - deltaR; // Hue
        
		if (hsl.x < 0.0)
			hsl.x += 1.0; // Hue
		else if (hsl.x > 1.0)
			hsl.x -= 1.0; // Hue
	}
    
	return hsl;
}

float HueToRGB(float f1, float f2, float hue)
{
	if (hue < 0.0)
		hue += 1.0;
	else if (hue > 1.0)
		hue -= 1.0;
	float res;
	if ((6.0 * hue) < 1.0)
		res = f1 + (f2 - f1) * 6.0 * hue;
	else if ((2.0 * hue) < 1.0)
		res = f2;
	else if ((3.0 * hue) < 2.0)
		res = f1 + (f2 - f1) * ((0.6666) - hue) * 6.0;
	else
		res = f1;
	return res;
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb;
	
	if (hsl.y == 0.0)
		rgb = vec3(hsl.z, hsl.z, hsl.z); // Luminance
	else
	{
		float f2;
		
		if (hsl.z < 0.5)
			f2 = hsl.z * (1.0 + hsl.y);
		else
			f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
        
		float f1 = 2.0 * hsl.z - f2;
		
		rgb.r = HueToRGB(f1, f2, hsl.x + (0.3333));
		rgb.g = HueToRGB(f1, f2, hsl.x);
		rgb.b= HueToRGB(f1, f2, hsl.x - (0.3333));
	}
	
	return rgb;
}

vec3 BlendHue(vec3 base, float blendHue)
{
	vec3 baseHSL = RGBToHSL(base);
	return HSLToRGB(vec3(blendHue, baseHSL.g, baseHSL.b));
}

void main() {
    float blendHue = 0.59574467;
    vec4 originalColor = texture2D(u_texture, v_texCoord);
    vec3 blendedColor = BlendHue(vec3(originalColor), blendHue);
    gl_FragColor = vec4(blendedColor, originalColor.a);
}